import { createStore } from 'vuex'
import dashboard from './modules/dashboard'
import { LOGIN, LOGIN_SUCCESS, LOGOUT, TITLE } from './mutation-types'

const store = createStore({
    state () {
        return {
            Session: {
                Token: null,
                User: null
            },
            Budgets: {},
        }
    },
    mutations: {
        initializeStore(state) {
            const store = localStorage.getItem("store");
            if(store){
                this.replaceState(
                    Object.assign(state, JSON.parse(store))
                );
            }
        },
        [TITLE](state, title) {
            document.title = "Budgeteer - " + title;
        },
        [LOGIN_SUCCESS](state, result) {
            state.Session = {
                User: result.User,
                Token: result.Token
            };
            for (const budget of result.Budgets) {
                state.Budgets[budget.ID] = budget 
            }
        },
        setBudgets (state, budgets) {
            state.Budgets = budgets;
        },
        addBudget(state, budget) {
            state.Budgets.push(budget);
        },
        [LOGOUT](state, token) {
            state.Session = { Token: null, User: null }
        }
    },
    actions: {
        [LOGIN]({state, commit}, login) {
            fetch("/api/v1/user/login", {method: "POST", body: JSON.stringify(login)})
                .then(x => x.json())
                .then(x => {
                    commit(LOGIN_SUCCESS, x);
                    this.$router.replace("/dashboard");
                })
        },
        YNAB({getters, dispatch}, formData) {
            return dispatch("POST", {path: "/budget/"+getters.CurrentBudget.ID+"/import/ynab", body: formData});
        },
        GET({getters}, {path}){
            return fetch("/api/v1" + path, {
                headers: getters.AuthHeaders,
            })
        },
        POST({getters}, {path, body}){
            return fetch("/api/v1" + path, {
                method: "POST",
                headers: getters.AuthHeaders,
                body: body,
            })
        }
    },
    getters: {
        Budgets(state) {
            return state.Budgets || [];
        },
        AuthHeaders(state) {
            return {
                'Authorization': 'Bearer ' + state.Session.Token
            }
        }
    },
    modules: {
        dashboard
    }
})

store.subscribe((mutation, state) => {
    switch(mutation.type){
        case "setCurrentBudget":
            store.dispatch("fetchBudget", mutation.payload.ID)
            break;
        case "setCurrentAccount":
            store.dispatch("fetchAccount", mutation.payload.ID)
            break;
    }
})

store.subscribe((mutation, state) => {
    let persistedState = {
        Session: state.Session,
        Budgets: state.Budgets,
        CurrentBudget: state.CurrentBudget
    }
    localStorage.setItem("store", JSON.stringify(persistedState));
})

export default store